/**
 * Copyright (c) 2018-present, heineiuo.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import { FileMetaData } from "./VersionFormat";
import Version from "./Version";
import VersionEdit from "./VersionEdit";
import { Options } from "./Options";
import Slice from "./Slice";
import SSTableBuilder from "./SSTableBuilder";
import { InternalKey, SequenceNumber } from "./Format";
import { FileHandle } from "./Env";
export default class Compaction {
    static targetFileSize(options: Options): number;
    static maxGrandParentOverlapBytes(options: Options): number;
    static totalFileSize(files: FileMetaData[]): number;
    static maxFileSizeForLevel(options: Options, level: number): number;
    constructor(options: Options, level: number);
    level: number;
    inputVersion: Version;
    grandparents: FileMetaData[];
    edit: VersionEdit;
    inputs: [FileMetaData[], FileMetaData[]];
    private grandparentIndex;
    private seenKey;
    private overlappedBytes;
    private _maxOutputFilesize;
    private levelPtrs;
    get maxOutputFilesize(): number;
    numInputFiles(which: 0 | 1): number;
    isTrivialMove(): boolean;
    isBaseLevelForKey(userKey: Slice): boolean;
    releaseInputs(): void;
    /**
     * Returns true if we should stop building the current output
     * before processing "internalKey".
     */
    shouldStopBefore(internalKey: Slice): boolean;
    addInputDeletions(edit: VersionEdit): void;
}
export declare class CompactionStats {
    times: number;
    bytesRead: number;
    bytesWritten: number;
    constructor();
    add(c: CompactionStats): void;
}
export interface CompactionStateOutput {
    number: number;
    fileSize: number;
    smallest: InternalKey;
    largest: InternalKey;
}
export declare class CompactionState {
    outputs: CompactionStateOutput[];
    smallestSnapshot: SequenceNumber;
    compaction: Compaction;
    outfile: FileHandle;
    builder: SSTableBuilder;
    totalBytes: number;
    constructor(c: Compaction);
    currentOutput(): CompactionStateOutput;
}
