/**
 * Copyright (c) 2018-present, heineiuo.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import { DatabaseOptions, ReadOptions, WriteOptions, IteratorOptions } from "./Options";
import { WriteBatch } from "./WriteBatch";
import { Snapshot } from "./Snapshot";
export default class Database {
    constructor(dbpath: string, options?: DatabaseOptions);
    private writers;
    private tmpBatch;
    private _internalKeyComparator;
    private _backgroundCompactionScheduled;
    private _dbpath;
    private _sn;
    private _ok;
    private _status;
    private _log;
    private _logFileNumber;
    private _memtable;
    private _immtable;
    private _versionSet;
    private _manualCompaction;
    private _bgError;
    private pendingOutputs;
    private _snapshots;
    private _stats;
    private _options;
    private _tableCache;
    private lockfile;
    private get userComparator();
    private shouldInit;
    private initVersionEdit;
    private recoverWrapper;
    private recover;
    private recoverLogFile;
    ok(): Promise<boolean>;
    close(): Promise<void>;
    iterator(options?: IteratorOptions): AsyncIterableIterator<{
        key: Uint8Array;
        value: Uint8Array;
    }>;
    /**
     * db.get -> memtable.get -> imm.get -> versionCurrent.get ->
     * versionCurrent.forEachOverlapping -> tableCache.get -> tableCache.findTable ->
     * table.get
     */
    get(userKey: string | Uint8Array, options?: ReadOptions): Promise<Uint8Array | void>;
    /**
     * TODO Trigger minor compaction's condition
     * 1. check if memtable bigger then 4mb
     * 2. check if this._immtable is not null（transfer memtable to sstable）
     */
    put(key: string | Uint8Array, value: string | Uint8Array, options?: WriteOptions): Promise<void>;
    del(key: string | Uint8Array, options?: WriteOptions): Promise<void>;
    batch(batch: WriteBatch, options?: WriteOptions): Promise<void>;
    getSnapshot(): Snapshot;
    releaseSnapshot(snapshot: Snapshot): void;
    private write;
    private buildBatchGroup;
    /**
     * if force is true, force compact
     */
    private makeRoomForWrite;
    private maybeScheduleCompaction;
    private backgroundCall;
    private backgroundCompaction;
    private doCompactionWork;
    private compactMemTable;
    private writeLevel0Table;
    private openCompactionOutputFile;
    private installCompactionResults;
    /**
     * manually compact
     */
    compactRange(begin: Uint8Array | string, end: Uint8Array | string): Promise<void>;
    private manualCompactRangeWithLevel;
    private manualCompactMemTable;
    private recordBackgroundError;
    private cleanupCompaction;
    private deleteObsoleteFiles;
    private finishCompactionOutputFile;
}
