/**
 * Copyright (c) 2018-present, heineiuo.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import { Buffer } from "./Buffer";
import Slice from "./Slice";
import { Comparator } from "./Comparator";
import BloomFilter from "./BloomFilter";
export declare enum FileType {
    kLogFile = 0,
    kDBLockFile = 1,
    kTableFile = 2,
    kDescriptorFile = 3,
    kCurrentFile = 4,
    kTempFile = 5,
    kInfoLogFile = 6
}
export declare enum ValueType {
    kTypeDeletion = 0,
    kTypeValue = 1
}
export declare const kValueTypeForSeek = ValueType.kTypeValue;
export declare type SequenceNumber = bigint;
export declare const kMaxSequenceNumber: SequenceNumber;
export declare class ParsedInternalKey {
    constructor(userKey?: Slice, sn?: SequenceNumber, valueType?: ValueType);
    userKey: Slice;
    sn: SequenceNumber;
    valueType: ValueType;
}
export declare enum VersionEditTag {
    kComparator = 1,
    kLogNumber = 2,
    kNextFileNumber = 3,
    kLastSequence = 4,
    kCompactPointer = 5,
    kDeletedFile = 6,
    kNewFile = 7,
    kPrevLogNumber = 9
}
export declare enum CompressionTypes {
    none = 0
}
export declare function extractUserKey(ikey: Slice): Slice;
export declare class InternalKey extends Slice {
    static kMaxSequenceNumber: SequenceNumber;
    static from(slice: Slice): InternalKey;
    constructor(userKey?: Slice, sn?: SequenceNumber, valueType?: ValueType);
    get userKey(): Slice;
    get type(): ValueType;
    get sequence(): number;
    decodeFrom(slice: Slice): boolean;
}
export declare class InternalKeyBuilder {
    build(sequence: SequenceNumber, valueType: ValueType, key: Slice): InternalKey;
}
export declare const kBlockTrailerSize = 5;
export declare class Config {
    static kNumLevels: number;
    static kL0CompactionTrigger: number;
    static kL0SlowdownWritesTrigger: number;
    static kL0StopWritesTrigger: number;
    static kMaxMemCompactLevel: number;
    static kReadBytesPeriod: number;
}
export declare const kSizeOfUInt32 = 4;
export declare class InternalKeyComparator implements Comparator {
    constructor(userComparator: Comparator);
    private _userComparator;
    private oneByte;
    get userComparator(): Comparator;
    getName(): string;
    findShortestSeparator(start: Slice, limit: Slice): void;
    findShortSuccessor(key: Slice): void;
    compare(key1: Slice, key2: Slice): number;
}
export declare function parseInternalKey(internalKey: Slice, ikey: ParsedInternalKey): boolean;
export declare class LookupKey {
    constructor(userKey: Slice, sequence: SequenceNumber);
    private _internalKeySizeBuf;
    private _userKeyBuf;
    private _sequenceBuf;
    get buffer(): Buffer;
    get internalKey(): Slice;
    get memKey(): Slice;
    get userKey(): Slice;
    set userKey(userKey: Slice);
}
export interface Entry {
    sequence?: SequenceNumber;
    type?: ValueType;
    key: Slice;
    value: Slice;
}
export interface EntryRequireType extends Entry {
    type: ValueType;
}
export interface Filter extends BloomFilter {
}
export declare class BlockHandle {
    static from(buf: Buffer): BlockHandle;
    offset: number;
    size: number;
    get buffer(): Buffer;
}
export interface MetaBlockEntry {
    name: string;
    handle: BlockHandle;
}
export interface DataBlockEntry {
    largest: Slice;
    handle: BlockHandle;
}
export interface BlockContents {
    data: Slice;
    cachable: boolean;
    heapAllocated: boolean;
}
