/**
 * Copyright (c) 2018-present, heineiuo.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import { Comparator } from "./Comparator";
import Slice from "./Slice";
import { Env, FileHandle } from "./Env";
import Block from "./SSTableBlock";
import Cache from "./Cache";
import { Buffer } from "./Buffer";
import { Snapshot } from "./Snapshot";
export interface FilterPolicy {
    name(): string;
    keyMayMatch(key: Slice, filter: Slice): boolean;
}
export interface ReadOptions {
    verifyChecksums?: boolean;
    fillCache?: boolean;
    snapshot?: Snapshot;
}
export declare const defaultReadOptions: Omit<Required<ReadOptions>, "snapshot">;
export interface IteratorOptions extends ReadOptions {
    reverse?: boolean;
    start?: string | Buffer;
}
export declare const defaultIteratorOptions: Omit<Required<IteratorOptions>, "snapshot">;
export interface WriteOptions {
    sync?: boolean;
}
export declare const defaultWriteOptions: Required<WriteOptions>;
export interface DatabaseOptions {
    comparator?: Comparator;
    writeBufferSize?: number;
    maxFileSize?: number;
    maxOpenFiles?: number;
    blockCache?: Cache<Buffer, Block>;
    blockSize?: number;
    blockRestartInterval?: number;
    reuseLogs?: boolean;
    filterPolicy?: FilterPolicy;
    debug?: boolean;
    lockfileStale?: number;
    env?: Env;
    log?: (message: string) => Promise<void>;
    infoLog?: FileHandle | null;
}
export declare type Options = Required<DatabaseOptions>;
export declare const defaultOptions: Omit<Required<Options>, "env">;
