/**
 * Copyright (c) 2018-present, heineiuo.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import Version from "./Version";
import Slice from "./Slice";
import Status from "./Status";
import { FileMetaData } from "./VersionFormat";
import VersionEdit from "./VersionEdit";
import { InternalKeyComparator, InternalKey, Entry } from "./Format";
import LogWriter from "./LogWriter";
import Compaction from "./Compaction";
import { Options } from "./Options";
import { TableCache } from "./SSTableCache";
interface RecoverResult {
    saveManifest?: boolean;
}
export default class VersionSet {
    compactPointers: Slice[];
    _current: Version;
    _dummyVersions: Version;
    hasLogNumber?: boolean;
    hasNextFileNumber?: boolean;
    hasPrevLogNumber?: boolean;
    logNumber: number;
    prevLogNumber: number;
    private _lastSequence;
    hasLastSequence?: boolean;
    manifestFileNumber: number;
    nextFileNumber: number;
    private _dbpath;
    _options: Options;
    internalKeyComparator: InternalKeyComparator;
    tableCache: TableCache;
    manifestWriter?: LogWriter;
    constructor(dbpath: string, options: Options, tableCache: TableCache, internalKeyComparator: InternalKeyComparator);
    get lastSequence(): bigint;
    set lastSequence(value: bigint);
    get current(): Version;
    getLevelSummary(): string;
    compactRange(level: number, begin: InternalKey, end: InternalKey): Compaction | void;
    private maxFileSizeForLevel;
    getNextFileNumber(): number;
    getNumLevelFiles(level: number): number;
    recover: () => Promise<RecoverResult>;
    markFileNumberUsed(num: number): void;
    private finalize;
    private getTotalBytes;
    logAndApply(edit: VersionEdit): Promise<Status>;
    needsCompaction(): boolean;
    /**
     * update this._current
     */
    private appendVersion;
    private reuseManifest;
    /**
     * dump current to manifest
     */
    private writeSnapshot;
    private writeCurrentFile;
    pickCompaction(): Compaction | void;
    /**
     * Stores the minimal range that covers all entries in inputs in
     * smallest, *largest.
     * REQUIRES: inputs is not empty
     */
    private getRange;
    /**
     * Stores the minimal range that covers all entries in inputs1 and inputs2
     * in *smallest, *largest.
     * REQUIRES: inputs is not empty
     */
    private getRange2;
    private findLargestKey;
    private addBoundaryInputs;
    findSmallestBoundaryFile(icmp: InternalKeyComparator, levelFiles: FileMetaData[], largestKey: InternalKey): FileMetaData;
    setupOtherInputs(c: Compaction): void;
    addLiveFiles(live: Set<number>): void;
    makeInputIterator(currentCompaction: Compaction): AsyncIterableIterator<Entry>;
    private levelFileEntryIterator;
}
export {};
